/*============================================================================
* PROJEKT:
* MODUL:    Main.c
* PROCESOR: Z8F Encore 1601
* VERZE:    1.0
* DATUM:    15.3.2004
* ===========================================================================
* Popis:  ridici logika
* ===========================================================================
*
*              PaPouch s.r.o. - www.papouch.com
*               
* ===========================================================================
*/

#include <ez8.h>
#include "spinel_z8_dma.h"      	// header modulu spinel
#include "main_spinel_z8_dma.h"    	// header modulu main



unsigned char volatile	cled_count;		// citac svitu CLED
#define	COMMANDLEN	30
unsigned char volatile  command[COMMANDLEN];
unsigned char 			cen = FALSE;	// priznak povoleni konfigurace
unsigned char volatile	status = 0x00;
unsigned char			wdt_res = FALSE;	// priznak povoleni nulovani WDT


/*============================================================================
  Set Status
  
  nastavi status
============================================================================*/
void set_status(void)
 {
  status = command[5];			// nastaveni statusu
  cs_sp_message(format, sig, ACK_OK, 0, 0); // vyslat OK
 }

/*============================================================================
  Read Status
  
  precte status
============================================================================*/
void read_status(void)
 {
  cs_sp_message(format, sig, ACK_OK, sizeof(status), &status); // vyslat status
 }

/*============================================================================
  Read Communication Error Counter
  
  vysle pocet chyb komunikace
============================================================================*/
void read_cec(void)
 {
  cs_sp_message(format, sig, ACK_OK, sizeof(commerr), &commerr); // vyslat pocet chyb
  commerr = 0x00;								// a nulovat pocet chyb
 }

/*============================================================================
  Reset 
  
  provede reset
============================================================================*/
void reset(void)
 {

  if(broadcast == FALSE)				// jeste posledni prani :-}}}
   {							// a splnit :-}}}
    cs_sp_message(format, sig, ACK_OK, 0, 0); // vyslat OK
   }
  DI();							// zavazat oci :-}}}
  while(1);						// a cekat na...
 }

/*============================================================================
 Zpracovani prijateho prikazu  
============================================================================*/
void proc_command(void)
 {
  unsigned char a;

  format = command[0];				// format prijate zpravy
  sig = command[2];					// podpis
  (command[1] == 0xff) ? (broadcast = TRUE) : (broadcast = FALSE);	// nastaveni priznaku BroadCast
  if(command[4] == C_CEN) 			// prikazu pro povoleni konfigurace
   {
    cen = TRUE;						// povolit konfiguraci
    cs_sp_message(format, sig, ACK_OK, 0, 0); // vyslat OK
   }
  else                              // zpracovani ostatnich prikazu
   {
      switch(command[4])
       {
        case C_NAME:
	     if(broadcast != TRUE)
	      send_name();
	     break;
	    case C_WSTAT:                	// prikaz "nastaveni statusu"
	     set_status();
	     break;
	    case C_RSTAT:                	// prikaz "cteni statusu"
	     if(broadcast == FALSE)			// nejedna se o BroadCast?
	      read_status();
	     break;
 	    case C_RCE:                  	// prikaz "cteni chyb komunikace"
	     if(broadcast == FALSE)			// nejedna se o BroadCast?
	      read_cec();
	     break;
	    case C_RESET:                  	// prikaz "reset"
	     reset();
	     break;
/*
	    case C_WCOM:                 	// prikaz "zapis komunikacnich parametru"
	     if(cen == TRUE)				// je povolena konfigurace?
	      write_c_cfg();				// je, zapsat
	     else
          cs_sp_message(format, sig, ACK_WD_AD, 0, 0);	// neni, vyslat chybu
	     break;
	    case C_RCOM:                 	// prikaz "cteni komunikacnich parametru"
	     if(broadcast == FALSE)			// nejedna se o BroadCast?
	      send_c_cfg();					// ne, precist
	     break;
/**/


   	    default:										// neplatny kod instrukce
         cs_sp_message(format, sig, ACK_BAD_I, 0, 0); 	// vyslat chybu
	     break;
       }
	cen = FALSE;					// zrusit priznak povoleni konfigurace
   }
  broadcast = FALSE;						// zruseni priznaku BroadCast
 }

/*------------------------------------------------------------------
 Timer0 Interrupt - 1ms

 pro CLED a rele
-------------------------------------------------------------------*/
#pragma interrupt
void isr_timer0(void)
 {

  if(cled_count >= CLED_TIME)
   C_CLED;
  else
   {
    if(cled_count == 0x00)
	 S_CLED;
	cled_count++;
   }

  wdt_res = TRUE;

  if(rswaitc != 0x00)
   rswaitc--;
 }

/*------------------------------------------------------------------
 Init

-------------------------------------------------------------------*/
void init(void)
 {

  WDTCTL = 0x55;		// WDT na 1s
  WDTCTL = 0xAA;
  WDTU = 0x01;
  WDTH = 0xc3;
  WDTL = 0x50;

  PAOUT = 0xff;			// nastavit vystupy
  PAAF = 0x30;         	// PA Alt Fun  0 = no AF
  PAOC = 0xff;          // PA Out Ctrl 0 = push-pull
  PADD = 0xfb;          // PA Data Dir 0 = output  
  PAADDR = 0x00; 	   	// PA CR No function

  T0CPH = 0x0D;
  T0CPL = 0x80;			// 1ms
  T0CTL = 0xe9;			// T0 Run, Continuous, 1:32

//  read_config();		// nacteni konfigurace

  commspeed = DEF_SPINSPD;		// DEBUG!!! default baudrate
  init_serial();

  SET_VECTOR(UART0_RX, isr_u0rx);      	// vektor preruseni prijmu
  SET_VECTOR(UART0_TX, isr_u0tx);      	// vektor preruseni vysilani
  SET_VECTOR(DMA, isr_DMA);				// vektor preruseni DMA
  SET_VECTOR(TIMER0, isr_timer0);		// vektor preruseni TIMER 0


  IRQ0E0 = 0x00;
  IRQ0E1 |= 0x30;   // UART0rx, T0
  IRQ2E0 = 0x00;
  IRQ2E1 |= 0x10;   // DMA
 }

void main(void)
 {
  unsigned int volatile p;
  register unsigned char i;

  S_CLED;
  DI();
  init();
  EI();
  C_CLED;

// ---------- DEBUG --------------
  format = 'a';				// format prijate zpravy
  sig = 'n';					// podpis
  broadcast = FALSE;
  send_name();
/**/


  while(1)
   {
    if(wdt_res == TRUE)			// je povoleno nulovani WDT?
	 {
      asm ("WDT");
	  wdt_res = FALSE;
	 }

    p = ((D0HIGH & 0x0f) << 8) + D0START;
    if((unsigned int)p_rx_h_buff != p)
     {
	  u0rx();
      if(spin_r_flags == RECEIVED)
	   { 
	    DI();
	    for(i = 0; i < COMMANDLEN + 1; i++)
	     command[i] = recstr[i];				// zkopirovat prijata data ke zpracovani - vzhledem k prijmu mimo interrupt toto nema moc smysl
	    EI();
	    spin_r_flags = NOREC;
	    proc_command();
	   }
	 }
   }
 }

